-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

-- Use the table provided for us
local ADDONNAME, RC = ...
RaidChecklist = RC

-- Pull in the localized strings
local L = RaidChecklistStrings
local P = RaidChecklistPetFamilies

if not RaidChecklistSupportedLocale then
  DEFAULT_CHAT_FRAME:AddMessage ("|cffff0000>> Raid Checklist is not supported for your locale ("..GetLocale()..") yet, sorry")
  RaidChecklist = nil
  return
end

local RCData = RaidChecklistData

RC.colorcodedbackdrop = true


-- We only generate the "available from" strings if they're actually used
local availablefrom = {}
setmetatable (availablefrom, {
  __index = function (table, tag)
    local s
    local last
    local NL = "\n "
    for _, entry in ipairs (RCData.table) do
      if entry.a == tag and last ~= entry.display then
        s = ((s and s..NL) or " ")..entry.display
        last = entry.display
      end
    end
    rawset (table, tag, s)
    return s
  end
})

-- No point recomputing these strings all the time, so pre-cache them
local unitstrings = {
  raid = { "player" }, -- This seems to be needed under certain circumstances. Odd.
  party = { "player" }, -- GetNumPartyMembers() excludes player!
  player = { "player" }
}
for i = 1,40 do table.insert (unitstrings.raid, "raid"..i) end
for i = 1,4  do table.insert (unitstrings.party, "party"..i) end

-- Find the unit string for the GUID, or nil if not found ("shouldn't happen")
local guid2unit = function (guid)
  local units
  if GetNumRaidMembers () > 0 then
    units = unitstrings.raid
  elseif GetNumPartyMembers () > 0 then
    units = unitstrings.party
  else
    units = unitstrings.player
  end
  for i,unit in ipairs (units) do
    if UnitExists (unit) and UnitGUID (unit) == guid then
      return unit
    end
  end
end

-- Inset constant
local INSET = 5

-- Convenience function to line up a frame into a parent, with insets
local function align_with_insets (alignto, f)
  f:ClearAllPoints ()
  f:SetPoint ("TOPLEFT", alignto, "TOPLEFT", INSET, -INSET)
  f:SetPoint ("BOTTOMRIGHT", alignto, "BOTTOMRIGHT", -INSET, INSET)
end

local function make_bordered_frame (parent, name)
  local f = CreateFrame ("Frame", name, parent)
  f:SetBackdrop ({
    edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
    edgeSize = 16,
    insets= { left = INSET, right = INSET, top = INSET, bottom = INSET }
  })
  f.texture = f:CreateTexture (nil, "BACKGROUND")
  f.texture:SetTexture (0, 0, 0, 0.75)
  align_with_insets (f, f.texture)
  return f
end


-- The button for anchoring and accessing the checklist
RC.button = CreateFrame ("Button", nil, UIParent)
RC.button:SetNormalTexture ("Interface\\AddOns\\RaidChecklist\\Textures\\button")
RC.button:SetHighlightTexture ("Interface\\AddOns\\RaidChecklist\\Textures\\button")
RC.button:SetPoint ("CENTER", 0, 0)
RC.button:SetWidth (24)
RC.button:SetHeight (24)
RC.button:SetMovable (true)
RC.button:SetUserPlaced (true)
RC.button:SetClampedToScreen (true)
RC.button.OnClick = function (self, button)
  if button == "RightButton" then
    InterfaceOptionsFrame_OpenToCategory (RC.settings_panel)
  else
    if RC.list:IsVisible () then
      RC.list:Hide ()
    else
      RC.list:Show ()
    end
  end
end
RC.button:SetScript ("OnClick", RC.button.OnClick)
RC.button:RegisterForClicks ("LeftButtonUp", "RightButtonUp")
RC.button:SetScript ("OnEvent", function (self, event, ...)
  if self[event] then
    self[event] (self, ...)
  end
end)


-- The main list frame
RC.list = make_bordered_frame (UIParent, "RaidChecklistList")
RC.list:SetPoint ("TOPLEFT", RC.button, "BOTTOMLEFT", 0, 0)
RC.list:Hide ()

-- A frame to hold the buff entries
RC.bufflist = CreateFrame ("Frame", nil, RC.list)
RC.bufflist:SetPoint ("TOPLEFT", RC.list, "TOPLEFT", INSET, -INSET)
RC.bufflist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

-- A frame to hold the debuff entries
RC.debufflist = CreateFrame ("Frame", nil, RC.list)
RC.debufflist:SetPoint ("TOPLEFT", RC.bufflist, "TOPRIGHT")
RC.debufflist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

-- A frame to hold the CC/PvP entries
RC.cclist = CreateFrame ("Frame", nil, RC.list)
RC.cclist:SetPoint ("TOPRIGHT", RC.list, "TOPRIGHT", -INSET, -INSET)
RC.cclist:SetPoint ("LEFT", RC.debufflist, "RIGHT")
RC.cclist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

-- A frame to hold the Hunter Pet entries
RC.hunterlist = CreateFrame ("Frame", nil, RC.list)
RC.hunterlist:SetPoint ("TOPLEFT", RC.list, "TOPLEFT", INSET, -INSET)
RC.hunterlist:SetPoint ("BOTTOM", RC.list, "BOTTOM")

if (RC.colorcodedbackdrop) then
  local texture = RC.bufflist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (0, 0, 1, 0.05)
  texture:SetAllPoints ()
  texture = RC.debufflist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (1, 0, 0, 0.05)
  texture:SetAllPoints ()
  texture = RC.cclist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (1, 1, 0, 0.05)
  texture:SetAllPoints ()
  texture = RC.hunterlist:CreateTexture (nil, "BACKGROUND")
  texture:SetTexture (0, 1, 0, 0.05)
  texture:SetAllPoints ()
end

-- The "haven't yet seen talents for" frame
RC.missing = make_bordered_frame (RC.list, "RaidChecklistMissing")
RC.missing:SetPoint ("TOPLEFT", RC.list, "BOTTOMLEFT")
RC.missing:SetPoint ("RIGHT", RC.list, "RIGHT")
RC.missing.hdr = RC.missing:CreateFontString (nil, "ARTWORK", "GameFontNormal")
RC.missing.hdr:SetPoint ("TOPLEFT", RC.missing, "TOPLEFT", INSET, -INSET)
RC.missing.hdr:SetPoint ("RIGHT", RC.missing, "RIGHT", -INSET, 0)
RC.missing.hdr:SetText (L.MISSINGTALENTS)
RC.missing.txt = RC.missing:CreateFontString (nil, "ARTWORK", "GameFontNormal")
RC.missing.txt:SetPoint ("TOPLEFT", RC.missing.hdr, "BOTTOMLEFT", INSET, 0)
RC.missing.txt:SetPoint ("RIGHT", RC.missing.hdr, "RIGHT")
RC.missing.txt:SetWordWrap (true)
RC.missing.txt:SetTextColor (0.7, 0.7, 0.7, 1)
RC.missing:SetScript ("OnEnter", function ()
  GameTooltip:SetOwner (RC.missing, "ANCHOR_RIGHT", 0, -RC.missing:GetHeight ())
  GameTooltip:SetText (L.INSPECTQUEUE)
  GameTooltip:AddLine (string.gsub (RC.missing_talents_for, ",", "\n"), 0.7, 0.7, 0.7, 1)
  if RC.missing.nowinspecting then
    GameTooltip:AddLine (L.NOWINSPECTING)
    GameTooltip:AddLine (RC.missing.nowinspecting, 0.7, 0.7, 0.7, 1)
  end
  GameTooltip:Show ()
end)
RC.missing:SetScript ("OnLeave", function () GameTooltip:Hide () end)
RC.missing:Hide ()

-- Inspection-in-progress notification
RC.missing.inspecting = CreateFrame ("Frame", nil, RC.missing)
RC.missing.inspecting:SetPoint ("TOPLEFT", RC.missing.hdr, "TOPLEFT")
RC.missing.inspecting:SetSize (12, 12)
RC.missing.inspecting.texture = RC.missing.inspecting:CreateTexture (nil, "ARTWORK")
RC.missing.inspecting.texture:SetTexture ("Interface\\AddOns\\RaidChecklist\\Textures\\light")
RC.missing.inspecting.texture:SetAllPoints ()
RC.missing.inspecting:Hide ()
RC.missing.inspecting:SetScript ("OnEvent", function (self, event, ...)
  if self[event] then
    self[event] (self, ...)
  end
end)
RC.missing.inspecting.INSPECT_READY = function (self)
  --UIFrameFlashStop (self)
  self:Hide ()
  RC.missing.nowinspecting = nil
end
RC.missing.inspecting:RegisterEvent ("INSPECT_READY")
hooksecurefunc("NotifyInspect", function (unit)
  -- UIFrameFlash causes taint apparently, grrr!
  RC.missing.inspecting:Show ()
  -- Flash for 2 minutes, and if we haven't cleared it by then, leave it on
  --UIFrameFlash (RC.missing.inspecting, 0, 1, 120, true, 0, 0)
  RC.missing.nowinspecting = UnitName (unit)
end)


-- Details frame
RC.detailshdr = make_bordered_frame (RC.list)
RC.detailshdr:SetPoint ("TOPLEFT", RC.list, "TOPRIGHT")
RC.detailshdr.txt = RC.detailshdr:CreateFontString (nil, "ARTWORK", "GameFontNormal")
align_with_insets (RC.detailshdr, RC.detailshdr.txt)
RC.detailshdr.txt:SetTextColor (1.0, 1.0, 1.0)
RC.detailshdr:SetHeight (2*INSET + 22)
RC.detailshdr:Hide ()
RC.detailshdr.closebtn = CreateFrame ("Button", nil, RC.detailshdr, "UIPanelCloseButton")
RC.detailshdr.closebtn:SetPoint ("TOPRIGHT", RC.detailshdr, "TOPRIGHT")
RC.details = make_bordered_frame (RC.detailshdr)
RC.details.hdr = RC.detailshdr
RC.details:SetPoint ("TOPLEFT", RC.detailshdr, "BOTTOMLEFT")
RC.details:SetPoint ("RIGHT", RC.detailshdr, "RIGHT")
RC.details:SetPoint ("BOTTOM", RC.list, "BOTTOM")
RC.details.txt = RC.details:CreateFontString (nil, "ARTWORK", "GameFontNormal")
RC.details.display = CreateFrame ("SimpleHTML", nil, RC.details)
RC.details.display:SetFontObject ("GameFontNormal")
RC.details.display:SetScript ("OnHyperlinkClick", ChatFrame_OnHyperlinkShow)
RC.details.display:SetPoint ("TOPLEFT", RC.details, "TOPLEFT", INSET, -INSET)
RC.details.display:SetPoint ("RIGHT", RC.details, "RIGHT", -INSET, 0)
RC.details.display:SetPoint ("BOTTOM", RC.details, "BOTTOM", 0, INSET)
RC.details.display:SetJustifyV ("TOP")
RC.details.display:SetJustifyH ("LEFT")
RC.details:EnableMouse (true)

local function GetSpellIconEscape (id)
  local _, _, t = GetSpellInfo (id)
  return "|T"..(t or "Interface\\Icons\\INV_Misc_QuestionMark")..":0|t"
end

local function GetTalentLink (id)
  local n = GetSpellInfo (id)
  return n and "|cff71d5ff|Hspell:"..id.."|h["..n.."]|h|r" or "|cffff0000?|r"
end

local DetailedAbilityInfo = {}
setmetatable (DetailedAbilityInfo, { __index = function (table, key)
  local s = L.DETAILS.."\n"
  for _, entry in ipairs (RCData.table) do
    if entry.a == key then
      s = s.."\n "
      local id
      if entry.t then id = entry.t end
      if entry.i then id = entry.i end
      if entry.r then id = entry.r end
      local icon = GetSpellIconEscape (id)
      if icon then s = s..icon.." " end

      s = s..(entry.c or entry.display) -- Cater for entry.e which has custom entry.display
      if entry.t then s = s.." "..GetTalentLink (entry.t)..", "..L.TALENT end
      if entry.i then s = s.." "..GetTalentLink (entry.i) end -- GetSpellLink doesn't work with specialization skills (e.g. Mangle), so create our own links instead
      if entry.r then s = s.." "..GetSpellLink (entry.r) end
      if entry.p then s = s..","..L.PET..entry.p end
      if entry.i and not entry.l and not entry.p and not entry.e then s = s ..", "..L.SPECIALIZATION end
      if entry.l then s = s..", "..L.LEVEL.." "..entry.l end
      if entry.s then s = s.." "..entry.s end
      if entry.e then s = s..", "..L.RACIAL end
    end
  end
  rawset (table, key, s)
  return s
end })
RC.details.ShowAbility = function (self, tag)
  self.hdr.txt:SetText (L[tag])
  local s = DetailedAbilityInfo[tag]
  self.display:SetText (s)
  self.current = tag
  self.txt:SetText (s)
  local width = self.txt:GetStringWidth () * 1.06 + INSET*2
  self.hdr:SetWidth (width)
  self.hdr:Show ()
end


-- FIXME: this is ugly as...
local function merge_providers ()
  local all = {}
  for k,v in pairs (RC.providers) do
    all[k] = {}
    for guid, plevel in pairs (v) do
      all[k][guid] = plevel
    end
  end
  for k,v in pairs (RC.petproviders) do
    for guid, plevel in pairs (v) do
      if plevel > (all[k][guid] or -1) then
        all[k][guid] = plevel
      end
    end
  end
  for k,v in pairs (RC.stableproviders) do
    for guid, plevel in pairs (v) do
      if plevel > (all[k][guid] or -1) then
        all[k][guid] = plevel
      end
    end
  end
  return all
end
RC.merge_providers = merge_providers


local function do_tooltip (frame)
  GameTooltip:SetOwner (frame, "ANCHOR_RIGHT", 0, -frame:GetHeight ())
  GameTooltip:SetText (L[frame.tag], 1, 1, 1, 1)
  GameTooltip:AddLine (L.AVAILABLEFROM)
  GameTooltip:AddLine (availablefrom[frame.tag])
  GameTooltip:AddLine ("\n"..L.PROVIDEDBY, 1, 0.82, 0, 1)
  local guidlist = merge_providers ()[frame.tag]
  if not next (guidlist) then
    GameTooltip:AddLine (" "..L.NOBODY, 0.7, 0.7, 0.7, 1)
  else
    local names
    for guid,_ in pairs (guidlist) do
      if names then names = names.."\n " else names = " " end
      names = names..(RC.provider_names[guid] or ("<missing name for "..guid..">"))
      local unit = guid2unit (guid)
      if unit and not UnitIsConnected (unit) then
        names = names.."|cffff0000 "..L.OFFLINE
      end
    end
    GameTooltip:AddLine (names, 1, 1, 1, 1)
  end
  GameTooltip:Show ()
end

local function hide_tooltip ()
  GameTooltip:Hide ()
end

local function toggle_details (frame)
  local details = RC.details
  if details:IsVisible () and details.current == frame.tag then
    details.hdr:Hide ()
  else
    details:ShowAbility (frame.tag)
  end
end


local function make_frame_entry (parent, relto, xoffs, extra_width, hilight, text, tag)
  local e = CreateFrame ("Frame", nil, parent)
  e:SetPoint ("TOPLEFT", relto, "BOTTOMLEFT")
  local txt = e:CreateFontString (nil, "ARTWORK", "GameFontNormal")
  e:SetWidth (1000) -- Give it enough space to give us a good estimate
  txt:SetPoint ("TOPLEFT", e, "TOPLEFT", xoffs, 0)
  txt:SetPoint ("RIGHT", e, "RIGHT")
  txt:SetJustifyH ("LEFT")
  txt:SetTextColor (0.7, 0.7, 0.7, 1)
  txt:SetText (text)

  e.desired_width = txt:GetStringWidth () * 1.07 + xoffs + extra_width -- x1.0.7 since it seems to underestimate :(
  e:SetHeight (txt:GetStringHeight ())
  e:SetPoint ("RIGHT", parent, "RIGHT") -- What we actually want
  e:EnableMouse (true)
  if hilight then
    local tex = e:CreateTexture (nil, "HIGHLIGHT")
    tex:SetAllPoints ()
    tex:SetTexture ("Interface\\Tooltips\\UI-Tooltip-Background") -- FIXME
    tex:SetAlpha (0.3)
    tex:SetVertexColor (1, 1, 0, 0.8)
    e:SetScript ("OnEnter", do_tooltip)
    e:SetScript ("OnLeave", hide_tooltip)
    e:SetScript ("OnMouseDown", toggle_details)

    local num = e:CreateFontString (nil, "ARTWORK", "GameFontNormal")
    num:SetPoint ("TOPRIGHT", e, "TOPRIGHT")
    num:SetJustifyH ("LEFT")
    num:SetTextColor (0.7, 0.7, 0.7, 1)
    e.num = num
  end
  e.txt = txt
  e.tag = tag
  return e
end


local function build_lists ()
  RC.buffhdr = make_frame_entry (RC.bufflist, RC.bufflist, 0, 0, false, L.BUFFS)
  RC.buffhdr:SetPoint ("TOPLEFT", RC.bufflist, "TOPLEFT") -- Reposition
  RC.buffhdr.txt:SetTextColor (1, 0.82, 0, 1)

  -- Calculate the necessary indentation required to allow for cardinality presentation to the left of each entry
  local temp = RC.buffhdr:CreateFontString (nil, "ARTWORK", "GameFontNormal")
  local cardinals = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, "MANY" }
  local card_width = 0
  for _, v in ipairs (cardinals) do
    temp:SetText (L[v] or "")
    card_width = max (card_width, temp:GetStringWidth ())
  end
  local NUMWIDTH = card_width
  local INDENT = 6

  RC.entries = {}
  local linkto = RC.buffhdr
  local maxwidth = RC.buffhdr.desired_width
  local totalheight = RC.buffhdr:GetHeight ()
  for i,b in ipairs (RCData.buffs) do
    local e = make_frame_entry (RC.bufflist, linkto, INDENT, NUMWIDTH, true, L[b], b)
    maxwidth = max (maxwidth, e.desired_width)
    totalheight = totalheight + e:GetHeight ()
    RC.entries[b] = e
    linkto = e
  end
  RC.bufflist:SetWidth (maxwidth)

  RC.debuffhdr = make_frame_entry (RC.debufflist, RC.buffhdr, 0, 0, false, L.DEBUFFS)
  RC.debuffhdr:SetPoint ("TOPLEFT", RC.buffhdr, "TOPRIGHT") -- Reposition to the right of Buffs list
  RC.debuffhdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.debuffhdr
  local maxwidth2 = RC.debuffhdr.desired_width
  for i,b in ipairs (RCData.debuffs) do
    local e = make_frame_entry (RC.debufflist, linkto, INDENT, NUMWIDTH, true, L[b], b)
    maxwidth2 = max (maxwidth2, e.desired_width)
    RC.entries[b] = e
    linkto = e
  end
  linkto = make_frame_entry (RC.debufflist, linkto, 0, 0, false, " ") -- Spacer
  RC.dispelhdr = make_frame_entry (RC.debufflist, linkto, 0, 0, false, L.DISPELS)
  RC.dispelhdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.dispelhdr
  for i,b in ipairs (RCData.dispels) do
    local e = make_frame_entry (RC.debufflist, linkto, INDENT, NUMWIDTH, true, L[b], b)
    maxwidth2 = max (maxwidth2, e.desired_width)
    RC.entries[b] = e
    linkto = e
  end
  RC.debufflist:SetWidth (maxwidth2)

  RC.cchdr = make_frame_entry (RC.cclist, RC.debuffhdr, 0, 0, false, L.CC)
  RC.cchdr:SetPoint ("TOPLEFT", RC.debuffhdr, "TOPRIGHT") -- Reposition to the right of Debuffs list
  RC.cchdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.cchdr
  local maxwidth3 = RC.cchdr.desired_width
  for i,b in ipairs (RCData.ccs) do
    local e = make_frame_entry (RC.cclist, linkto, INDENT, NUMWIDTH, true, L[b], b)
    maxwidth3 = max (maxwidth3, e.desired_width)
    RC.entries[b] = e
    linkto = e
  end
  linkto = make_frame_entry (RC.cclist, linkto, 0, 0, false, " ") -- Spacer
  RC.mischdr = make_frame_entry (RC.cclist, linkto, 0, 0, false, L.MISC)
  RC.mischdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.mischdr
  for i,b in ipairs (RCData.miscs) do
    local e = make_frame_entry (RC.cclist, linkto, INDENT, NUMWIDTH, true, L[b], b)
    maxwidth3 = max (maxwidth3, e.desired_width)
    RC.entries[b] = e
    linkto = e
  end
  RC.cclist:SetWidth (maxwidth3)

  RC.hunterhdr = make_frame_entry (RC.hunterlist, RC.hunterlist, 0, 0, false, L.HUNTERPETABILITIES)
  RC.hunterhdr:SetPoint ("TOPLEFT", RC.bufflist, "TOPLEFT") -- Reposition
  RC.hunterhdr.txt:SetTextColor (1, 0.82, 0, 1)
  linkto = RC.hunterhdr
  RC.hunterentries = {} -- Same function as RC.entries, but need separate since we duplicate keys
  local maxwidthhunter = RC.hunterhdr.desired_width
  for i,b in ipairs (RCData.hunterpetabilities) do
    local e = make_frame_entry (RC.hunterlist, linkto, INDENT, NUMWIDTH, true, L[b], b)
    maxwidthhunter = max (maxwidthhunter, e.desired_width)
    RC.hunterentries[b] = e
    linkto = e
  end
  RC.hunterlist:SetWidth (maxwidthhunter)

  RC.normal_width = maxwidth + maxwidth2 + maxwidth3 + 2 * INSET
  RC.hunter_width = maxwidthhunter + 2 * INSET

  RC.list:SetWidth (RC.normal_width)
  RC.list:SetHeight (totalheight + 3 * INSET) -- Looks better with a bit of extra space
end

build_lists ()


-- Levels: 0 = missing, 1 = offline, 2 = filtered/could-be-provided, 3 = provided
-- This function reduces the plevel to 1 or 2 as appropriate
local providerlevel4guid = function (guid, plevel)
  local db = RC.db
  local level = plevel
  local unit = guid2unit (guid)
  if unit and level > 0 then
    local israid, _, idx = string.find (unit, "raid(%d+)")
    if israid then
      local group = "group"..select(3, GetRaidRosterInfo (idx))
      if not db.profile[group] then
        level = 2
      end
      group = nil
    end
    if not UnitIsConnected (unit) then
      level = 1
    end
  else
    level = 0
  end
  return level
end

local providerlevel = function (guidlist)
  local highestlevel = 0
  local count = 0
  local playerprovided = false
  for guid, plevel in pairs (guidlist) do
    local level = providerlevel4guid (guid, plevel)
    if level > highestlevel then highestlevel = level end
    count = count + 1
    if guid2unit (guid) == "player" then
      playerprovided = true
    end
  end
  return highestlevel, count, playerprovided
end

-- Updates the color on each of the buff entries in the list
RC.OnRefresh = function (self)
  local all = self.merge_providers ()
  local colors = {
    [0] = { r = 0.7, g = 0.7, b = 0.7 },
    [1] = { r = 0.7, g = 0.0, b = 0.0 },
    [2] = { r = 0.0, g = 0.4, b = 1.0 },
    [3] = { r = 0.0, g = 1.0, b = 0.0 }
  }
  for key,guidlist in pairs (all) do
    local plevel, count, playerprovided = providerlevel (guidlist)
    local c = colors[plevel]
    self.entries[key].txt:SetTextColor (c.r, c.g, c.b, 1)
    self.entries[key].num:SetTextColor (c.r, c.g, c.b, 1)
    -- Handle non-arabic numerals (in theory at least) by translating 0-9,"MANY"
    local cardinal = L[count]
    if count > 9 then cardinal = L["MANY"] end
    -- Flag (literally) abilities that only the player provides
    if count == 1 and playerprovided then
      local flagcolor
      if plevel == 3 then
        flagcolor = "green"
      else
        flagcolor = "blue"
      end
      cardinal = "|TInterface\\AddOns\\RaidChecklist\\Textures\\"..flagcolor.."flag:0|t"
    end
    self.entries[key].num:SetText (cardinal)
  end
  all = nil

  -- Show players we haven't seen talents for yet, if there are any
  if self.missing_talents_for then
    self.missing.txt:SetText (string.gsub (self.missing_talents_for, ",", ", "))
    self.missing:SetHeight (self.missing.hdr:GetHeight () * 2 + self.missing.txt:GetHeight ())
    self.missing:Show ()
  else
    self.missing:Hide ()
  end

  -- Copy color and cardinality for the hunter list
  for key, entry in pairs (self.hunterentries) do
    local e = self.entries[key]
    entry.txt:SetTextColor (e.txt:GetTextColor ())
    entry.num:SetTextColor (e.num:GetTextColor ())
    entry.num:SetText (e.num:GetText ())
  end
end


RC.OnConfigChange = function (self)
  local db = RC.db
  if db.profile.show_anchor then
    self.button:Show ()
  else
    self.button:Hide ()
  end

  if db.profile.anchor_locked then
    self.button:SetScript ("OnMouseDown", nil)
    self.button:SetScript ("OnMouseUp", nil)
  else
    self.button:SetScript ("OnMouseDown", function (f) f:StartMoving () end)
    self.button:SetScript ("OnMouseUp", function (f)
      f:StopMovingOrSizing ()
      db.profile.anchorpoint, _, db.profile.relpoint, db.profile.x, db.profile.y = f:GetPoint (1)
    end)
  end

  self.list:ClearAllPoints ()
  self.missing:ClearAllPoints ()
  self.missing:SetPoint ("RIGHT", self.list, "RIGHT")
  local dir = db.profile.open_direction
  if dir == "DOWNRIGHT" then
    self.list:SetPoint ("TOPLEFT", self.button, "BOTTOMLEFT", 0, 0)
    self.missing:SetPoint ("TOPLEFT", self.list, "BOTTOMLEFT")
  elseif dir == "DOWNLEFT" then
    self.list:SetPoint ("TOPRIGHT", self.button, "BOTTOMRIGHT", 0, 0)
    self.missing:SetPoint ("TOPLEFT", self.list, "BOTTOMLEFT")
  elseif dir == "UPRIGHT" then
    self.list:SetPoint ("BOTTOMLEFT", self.button, "TOPLEFT", 0, 0)
    self.missing:SetPoint ("BOTTOMLEFT", self.list, "TOPLEFT")
  else -- UPLEFT
    self.list:SetPoint ("BOTTOMRIGHT", self.button, "TOPRIGHT", 0, 0)
    self.missing:SetPoint ("BOTTOMLEFT", self.list, "TOPLEFT")
  end

  self.detailshdr:ClearAllPoints ()
  if db.profile.details_side == "RIGHT" then
    self.detailshdr:SetPoint ("TOPLEFT", self.list, "TOPRIGHT")
  else
    self.detailshdr:SetPoint ("TOPRIGHT", self.list, "TOPLEFT")
  end

  self.button:ClearAllPoints ()
  self.button:SetPoint (db.profile.anchorpoint, UIParent, db.profile.relpoint, db.profile.x, db.profile.y)

  local fontfile, fontsize = self.entries.CRIT.txt:GetFont ()
  if (fontsize < 11 and db.profile.font_size == "normal") or (fontsize > 11 and db.profile.font_size == "small") then
    local grow = db.profile.font_size == "normal"
    if db.profile.font_size == "normal" then
      fontsize = 12
    elseif db.profile.font_size == "small" then
      fontsize = 10
    end

    for _,e in pairs (RC.entries) do
      e.txt:SetFont (fontfile, fontsize)
      e:SetHeight (fontsize)
      e.num:SetFont (fontfile, fontsize)
    end
    for _, e in pairs (RC.hunterentries) do
      e.txt:SetFont (fontfile, fontsize)
      e:SetHeight (fontsize)
      e.num:SetFont (fontfile, fontsize)
    end
    local hdrs = { RC.buffhdr, RC.debuffhdr, RC.dispelhdr, RC.cchdr, RC.mischdr, RC.hunterhdr }
    for _,e in pairs (hdrs) do
      e.txt:SetFont (fontfile, fontsize)
      e:SetHeight (fontsize)
    end

    local frames = { RC.bufflist, RC.debufflist, RC.cclist, RC.hunterlist, RC.list, RC.detailshdr }
    if grow then
      for _, frame in ipairs (frames) do
        frame:SetWidth (frame:GetWidth () * 1.2)
      end
      RC.normal_width = RC.normal_width * 1.2
      RC.hunter_width = RC.hunter_width * 1.2
      RC.list:SetHeight (RC.list:GetHeight () * 1.2)
    else
      for _, frame in ipairs (frames) do
        frame:SetWidth (frame:GetWidth () / 1.2)
      end
      RC.normal_width = RC.normal_width / 1.2
      RC.hunter_width = RC.hunter_width / 1.2
      RC.list:SetHeight (RC.list:GetHeight () / 1.2)
    end
  end

  if db.profile.hunterpet_mode then
    RC.hunterlist:Show ()
    RC.bufflist:Hide ()
    RC.debufflist:Hide ()
    RC.cclist:Hide ()
    RC.list:SetWidth (RC.hunter_width)
  else
    RC.hunterlist:Hide ()
    RC.bufflist:Show ()
    RC.debufflist:Show ()
    RC.cclist:Show ()
    RC.list:SetWidth (RC.normal_width)
  end

  frames = { self.list, self.missing, self.detailshdr, self.details }
  for _, frame in ipairs (frames) do
    frame.texture:SetTexture (0, 0, 0, db.profile.opacity);
  end

  self.detailshdr.txt:SetFont (fontfile, fontsize)
  self.details.txt:SetFont (fontfile, fontsize)
  self.details.display:SetFont (fontfile, fontsize)

  self:OnRefresh () -- In case the filtered groups changed
end


-- Enable /rcl and /raidchecklist for toggling the checklist
SlashCmdList.RAIDCHECKLIST = function ()
  if RC.list:IsVisible () then
    RC.list:Hide ()
  else
    RC.list:Show ()
  end
end
SLASH_RAIDCHECKLIST1 = "/rcl"
SLASH_RAIDCHECKLIST2 = "/raidchecklist"

